]> git.sesse.net Git - movit/blobdiff - effect.h
Handle texture non-bounce a bit better.
[movit] / effect.h
index c911f276ef71d94a6ab281f6339c87bb702af806..9d95705f6a66ce344a54bd18eb83154018d85c35 100644 (file)
--- a/effect.h
+++ b/effect.h
@@ -1,5 +1,5 @@
-#ifndef _EFFECT_H
-#define _EFFECT_H 1
+#ifndef _MOVIT_EFFECT_H
+#define _MOVIT_EFFECT_H 1
 
 // Effect is the base class for every effect. It basically represents a single
 // GLSL function, with an optional set of user-settable parameters.
 // effect instance; use the macro PREFIX() around your identifiers to
 // automatically prepend that prefix.
 
+#include <epoxy/gl.h>
+#include <assert.h>
+#include <stddef.h>
 #include <map>
 #include <string>
-#include <vector>
-
-#include <assert.h>
 
-#include <Eigen/Core>
+#include "defs.h"
 
-#include <GL/glew.h>
-#include "util.h"
+namespace movit {
 
 class EffectChain;
 class Node;
@@ -41,23 +40,13 @@ struct RGBTriplet {
 };
 
 // Can alias on a float[4].
-struct RGBATriplet {
-       RGBATriplet(float r, float g, float b, float a)
+struct RGBATuple {
+       RGBATuple(float r, float g, float b, float a)
                : r(r), g(g), b(b), a(a) {}
 
        float r, g, b, a;
 };
 
-// Convenience functions that deal with prepending the prefix.
-GLint get_uniform_location(GLuint glsl_program_num, const std::string &prefix, const std::string &key);
-void set_uniform_int(GLuint glsl_program_num, const std::string &prefix, const std::string &key, int value);
-void set_uniform_float(GLuint glsl_program_num, const std::string &prefix, const std::string &key, float value);
-void set_uniform_vec2(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values);
-void set_uniform_vec3(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values);
-void set_uniform_vec4(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values);
-void set_uniform_vec4_array(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const float *values, size_t num_values);
-void set_uniform_mat3(GLuint glsl_program_num, const std::string &prefix, const std::string &key, const Eigen::Matrix3d &matrix);
-
 class Effect {
 public:
        virtual ~Effect() {}
@@ -99,7 +88,8 @@ public:
        // This is the most natural format for processing, and the default in
        // most of Movit (just like linear light is).
        //
// If you set INPUT_AND_OUTPUT_ALPHA_PREMULTIPLIED, all of your inputs
        // (if any) are guaranteed to also be in premultiplied alpha.
        // Otherwise, you can get postmultiplied or premultiplied alpha;
        // you won't know. If you have multiple inputs, you will get the same
@@ -113,22 +103,32 @@ public:
                // pre- and postmultiplied.
                OUTPUT_BLANK_ALPHA,
 
+               // Always outputs postmultiplied alpha. Only appropriate for inputs.
+               OUTPUT_POSTMULTIPLIED_ALPHA,
+
                // Always outputs premultiplied alpha. As noted above,
                // you will then also get all inputs in premultiplied alpha.
                // If you set this, you should also set needs_linear_light().
-               INPUT_AND_OUTPUT_ALPHA_PREMULTIPLIED,
+               INPUT_AND_OUTPUT_PREMULTIPLIED_ALPHA,
 
-               // Always outputs postmultiplied alpha. Only appropriate for inputs.
-               OUTPUT_ALPHA_POSTMULTIPLIED,
+               // Like INPUT_AND_OUTPUT_PREMULTIPLIED_ALPHA, but also guarantees
+               // that if you get blank alpha in, you also keep blank alpha out.
+               // This is a somewhat weaker guarantee than DONT_CARE_ALPHA_TYPE,
+               // but is still useful in many situations, and appropriate when
+               // e.g. you don't touch alpha at all.
+               //
+               // Does not make sense for inputs.
+               INPUT_PREMULTIPLIED_ALPHA_KEEP_BLANK,
 
// Keeps the type of alpha unchanged from input to output.
// Usually appropriate if you process all color channels
// in a linear fashion, and do not change alpha.
                //
                // Does not make sense for inputs.
                DONT_CARE_ALPHA_TYPE,
        };
-       virtual AlphaHandling alpha_handling() const { return INPUT_AND_OUTPUT_ALPHA_PREMULTIPLIED; }
+       virtual AlphaHandling alpha_handling() const { return INPUT_AND_OUTPUT_PREMULTIPLIED_ALPHA; }
 
        // Whether this effect expects its input to come directly from
        // a texture. If this is true, the framework will not chain the
@@ -167,6 +167,15 @@ public:
        // next effect set needs_texture_bounce()).
        virtual bool changes_output_size() const { return false; }
 
+       // Whether this effect is effectively sampling from a a single texture.
+       // If so, it will override needs_texture_bounce(); however, there are also
+       // two demands it needs to fulfill:
+       //
+       //  1. It needs to be an Input, ie. num_inputs() == 0.
+       //  2. It needs to allocate exactly one sampler in set_gl_state(),
+       //     and allow dependent effects to change that sampler state.
+       virtual bool is_single_texture() const { return false; }
+
        // If changes_output_size() is true, you must implement this to tell
        // the framework what output size you want. Also, you can set a
        // virtual width/height, which is the size the next effect (if any)
@@ -198,6 +207,12 @@ public:
        // if you have several, they will be INPUT1(), INPUT2(), and so on.
        virtual unsigned num_inputs() const { return 1; }
 
+       // Inform the effect that it has been just added to the EffectChain.
+       // The primary use for this is to store the ResourcePool uesd by
+       // the chain; for modifications to it, rewrite_graph() below
+       // is probably a better fit.
+       virtual void inform_added(EffectChain *chain) {}
+
        // Let the effect rewrite the effect chain as it sees fit.
        // Most effects won't need to do this, but this is very useful
        // if you have an effect that consists of multiple sub-effects
@@ -259,31 +274,14 @@ protected:
        void register_vec3(const std::string &key, float *values);
        void register_vec4(const std::string &key, float *values);
 
-       // This will register a 1D texture, which will be bound to a sampler
-       // when your GLSL code runs (so it corresponds 1:1 to a sampler2D uniform
-       // in GLSL).
-       //
-       // Note that if you change the contents of <values>, you will need to
-       // call invalidate_1d_texture() to have the picture re-uploaded on the
-       // next frame. This is in contrast to all the other parameters, which are
-       // set anew every frame.
-       void register_1d_texture(const std::string &key, float *values, size_t size);
-       void invalidate_1d_texture(const std::string &key);
-       
 private:
-       struct Texture1D {
-               float *values;
-               size_t size;
-               bool needs_update;
-               GLuint texture_num;
-       };
-
        std::map<std::string, int *> params_int;
        std::map<std::string, float *> params_float;
        std::map<std::string, float *> params_vec2;
        std::map<std::string, float *> params_vec3;
        std::map<std::string, float *> params_vec4;
-       std::map<std::string, Texture1D> params_tex_1d;
 };
 
-#endif // !defined(_EFFECT_H)
+}  // namespace movit
+
+#endif // !defined(_MOVIT_EFFECT_H)
@@ -99,7 +88,8 @@ public:
        // most of Movit (just like linear light is).
        //
-       // If you set INPUT_AND_OUTPUT_ALPHA_PREMULTIPLIED, all of your inputs
+       // If you set INPUT_AND_OUTPUT_PREMULTIPLIED_ALPHA or
+       // INPUT_PREMULTIPLIED_ALPHA_KEEP_BLANK, all of your inputs
        // (if any) are guaranteed to also be in premultiplied alpha.
        // Otherwise, you can get postmultiplied or premultiplied alpha;
@@ -113,22 +103,32 @@ public:
 
-               // Keeps the type of alpha unchanged from input to output.
-               // Usually appropriate if you process all color channels
-               // in a linear fashion, and do not change alpha.
+               // Keeps the type of alpha (premultiplied, postmultiplied, blank)
+               // unchanged from input to output. Usually appropriate if you
+               // process all color channels in a linear fashion, do not change
+               // alpha, and do not produce any new pixels thare have alpha != 1.0.
                //
                // Does not make sense for inputs.